//-------------------------------------------------------------------------------------
//Simple pass through of the input clock just for simulation
//In real design the actual PLL IP is needed

module pll
(
  input  wire clk,
  input  wire reset,
  output wire clk0_out
);

  assign clk0_out = clk;

endmodule

//-------------------------------------------------------------------------------------
//A simple clock divider module for which the width and the last count can be set

module clockdivider
(
  input  wire clk_in,
  output reg clk_out = 0
);
parameter WIDTH = 32;
parameter LAST  = (2 ** WIDTH) - 1;

reg [WIDTH-1:0] count = 0;

  always @(posedge clk_in)
    begin
      if(count == LAST)
        count <= 0;
      else
        count <= count + 1;
    end

  always @(posedge clk_in)
    begin
      if(count == LAST)
        clk_out <= ~clk_out;
    end

endmodule

//-------------------------------------------------------------------------------------
//A 16 bit pulse width counter module with the pulse width setting as an input

module pwmcounter
(
  input wire clk,

  input wire [15:0] pulsewidth,

  output reg pwm = 0
);
parameter LAST = 65535;

reg [15:0] count = 0;

  always @(posedge clk)
    begin
      if(count == LAST)
        count <= 0;
      else
        begin
          count <= count + 1;

          pwm <= count < pulsewidth;
        end
    end

endmodule

//-------------------------------------------------------------------------------------

module fnirsi_1013D
(
  //Input signals
  input wire       i_xtal,       //50 MHz clock
  input wire       i_mcu_clk,    //Active low going pulse from the mcu to clock data
  input wire       i_mcu_rws,    //Read 0 / write 1
  input wire       i_mcu_dcs,    //Data 0 / command 1
  input wire [7:0] i_adc1A_d,    //ADC 1 databus
  input wire [7:0] i_adc1B_d,        
  input wire [7:0] i_adc2A_d,    //ADC 2 databus        
  input wire [7:0] i_adc2B_d,        

  //Bi-directional parallel data bus to / from the mcu        
  inout wire [7:0] io_mcu_data,

  //Outputs
  output wire o_adc1_encA,       //ADC clock outputs   
  output wire o_adc1_encB,
  output wire o_adc2_encA,        
  output wire o_adc2_encB,        
  output wire o_offset_1,        //DC offset channnel 1
  output wire o_relay1_1,        //Relay channel 1
  output wire o_relay1_2,        
  output wire o_relay1_3,
  output wire o_ac_dc_1,         //AC / DC select channel 1        
  output wire o_offset_2,        //DC offset channnel 2
  output wire o_relay2_1,        //Relay channel 2
  output wire o_relay2_2,        
  output wire o_relay2_3,
  output wire o_ac_dc_2,         //AC / DC select channel 2  
  output wire o_pwm_display,     //Display brightness control
  output wire o_1khz_calib
);

//-------------------------------------------------------------------------------------
//Wires to connect parts together

  wire clk_200MHz;
  wire clk_50MHz;
  wire clk_1KHz;

  wire sample_clock_enable;

//-------------------------------------------------------------------------------------
//Registers to store working data

  reg [7:0] mcu_command = 0;           //Current command set by the MCU

  reg [7:0] o_mcu_data = 0;            //Data set to be read by the MCU

  reg [3:0] mcu_state = 0;             //Current state for writing or reading multiple bytes (Can be reduced to two bits)

  reg [15:0] ch1_offset = 0;           //Channel 1 DC offset setting
  reg [15:0] ch2_offset = 0;           //Channel 2 DC offset setting

  reg [15:0] display_brigthness = 0;   //Display brightness control

  reg [15:0] touch_panel_address = 0;  //Data written with command 0x3C and read with 0x41

  reg [31:0] sample_rate_divider = 0;  //For controlling the sample rate a large divider

  reg [31:0] time_base_count = 0;      //???

  reg relay1_1 = 0;                    //Relay control for channel 1
  reg relay1_2 = 0;
  reg relay1_3 = 0;
  reg ac_dc_1  = 0;

  reg relay2_1 = 0;                    //Relay control for channel 2
  reg relay2_2 = 0;
  reg relay2_3 = 0;
  reg ac_dc_2  = 0;

  reg [31:0] sample_rate_counter = 0;  //Counter for making the sample rate

  reg sample_write_clock = 0;          //Free running clock on which the samples are taken

  reg adc1_encA = 0;                   //Clock signals for the ADC's
  reg adc1_encB = 0;
  reg adc2_encA = 0;
  reg adc2_encB = 0;

//-------------------------------------------------------------------------------------

  pll main_clock
  (
    .clk      (i_xtal),
    .reset    (0),
    .clk0_out (clk_200MHz)
  );

//-------------------------------------------------------------------------------------
//Clock divider for making the 50MHz clock
//The original uses 16 bits

  clockdivider #
  (
    .WIDTH (1),
    .LAST  (1)
  )
  peripheral_clock
  (
    .clk_in  (clk_200MHz),
    .clk_out (clk_50MHz)
  );

//-------------------------------------------------------------------------------------
//Clock divider for making the 1KHz calibration signal
//The original uses 32 bits and 25000 for last which gives the incorrect frequency

  clockdivider #
  (
    .WIDTH (16),
    .LAST  (24999)
  )
  calibration_clock
  (
    .clk_in  (clk_50MHz),
    .clk_out (clk_1KHz)
  );

  assign o_1khz_calib = clk_1KHz;

//-------------------------------------------------------------------------------------
//Channel 1 DC offset

  pwmcounter #
  (
    .LAST (1999)
  )
  channel1_dc_offset
  (
    .clk        (clk_50MHz),
    .pulsewidth (ch1_offset),
    .pwm        (o_offset_1)
  );

//-------------------------------------------------------------------------------------
//Channel 2 DC offset

  pwmcounter #
  (
    .LAST (1999)
  )
  channel2_dc_offset
  (
    .clk        (clk_50MHz),
    .pulsewidth (ch2_offset),
    .pwm        (o_offset_2)
  );

//-------------------------------------------------------------------------------------
//Display brightness control

  pwmcounter #
  (
    .LAST (59999)
  )
  display_brightness
  (
    .clk        (clk_50MHz),
    .pulsewidth (display_brigthness),
    .pwm        (o_pwm_display)
  );

//-------------------------------------------------------------------------------------
//Sample write clock

  always @(negedge clk_200MHz)
    begin
      if(sample_clock_enable == 1)
        begin
          sample_write_clock <= ~sample_write_clock;

          adc1_encB = ~adc1_encB;
          adc2_encB = ~adc2_encB;

          //The A and B clocks are inverted versions of each other
          adc1_encA <= ~adc1_encB;
          adc2_encA <= ~adc2_encB;
        end
    end

  always @(negedge clk_200MHz)
    begin
      if(sample_clock_enable == 1)
        sample_rate_counter = 0;
      else
        sample_rate_counter = sample_rate_counter + 1;
    end

  assign sample_clock_enable = sample_rate_divider < (sample_rate_counter + 1);

//-------------------------------------------------------------------------------------
//Loading of the command register

  always @(posedge i_mcu_clk)
    begin
      if(i_mcu_dcs && i_mcu_rws)
        mcu_command <= io_mcu_data;
    end

//-------------------------------------------------------------------------------------
//State machine for the multiple byte write and read of data
//Only used when writing data for the commands 0x0D, 0x0E, 0x1F, 0x32, 0x35, 0x38 and 0x3C
//Or when reading data for the commands 0x06, 0x14 and 0x41

  always @(posedge i_mcu_clk)
    begin
      if(i_mcu_dcs == 1'b1)
        mcu_state <= 3'h0;
      else if((((mcu_command == 8'h0D) || (mcu_command == 8'h0E) || (mcu_command == 8'h1F) || (mcu_command == 8'h32) || (mcu_command == 8'h35) || (mcu_command == 8'h38) || (mcu_command == 8'h3C)) && i_mcu_rws) ||
              (((mcu_command == 8'h06) || (mcu_command == 8'h14) || (mcu_command == 8'h41)) && ~i_mcu_rws))
        mcu_state <= mcu_state + 1;
    end

//-------------------------------------------------------------------------------------
//Loading data into the registers based on the set command

  always @(posedge i_mcu_clk)
    begin
      case(mcu_command)
        //Loading of the four bytes of the sample rate divider high byte first
        8'h0D:
          begin
            case(mcu_state)
              4'h0:
                sample_rate_divider[31:24] <= io_mcu_data;
              4'h1:
                sample_rate_divider[23:16] <= io_mcu_data;
              4'h2:
                sample_rate_divider[15:8] <= io_mcu_data;
              4'h3:
                sample_rate_divider[7:0] <= io_mcu_data;
            endcase
          end

        //Loading of the four bytes of the time base counter high byte first
        8'h0E:
          begin
            case(mcu_state)
              4'h0:
                time_base_count[31:24] <= io_mcu_data;
              4'h1:
                time_base_count[23:16] <= io_mcu_data;
              4'h2:
                time_base_count[15:8] <= io_mcu_data;
              4'h3:
                time_base_count[7:0] <= io_mcu_data;
            endcase
          end

        //Loading of the two bytes of the channel 1 DC offset high byte first
        8'h32:
          begin
            case(mcu_state)
              4'h0:
                ch1_offset[15:8] <= io_mcu_data;
              4'h1:
                ch1_offset[7:0] <= io_mcu_data;
            endcase
          end

        //Controlling of the channel 1 input sensitivity relays
        8'h33:
          begin
            if(io_mcu_data < 8'h6)
              begin
                relay1_1 <=  io_mcu_data[0];
                relay1_2 <= ~io_mcu_data[1];
                relay1_3 <= ~io_mcu_data[2];
              end
          end

        //Controlling of the channel 1 input for AC or DC
        8'h34:
          ac_dc_1 <= io_mcu_data[0];

        //Loading of the two bytes of the channel 2 DC offset high byte first
        8'h35:
          begin
            case(mcu_state)
              4'h0:
                ch2_offset[15:8] <= io_mcu_data;
              4'h1:
                ch2_offset[7:0] <= io_mcu_data;
            endcase
          end

        //Controlling of the channel 2 input sensitivity relays
        8'h36:
          begin
            if(io_mcu_data < 8'h6)
              begin
                relay2_1 <=  io_mcu_data[0];
                relay2_2 <= ~io_mcu_data[1];
                relay2_3 <= ~io_mcu_data[2];
              end
          end

        //Controlling of the channel 2 input for AC or DC
        8'h37:
          ac_dc_2 <= io_mcu_data[0];

        //Loading of the two bytes of the display brightness control high byte first
        8'h38:
          begin
            case(mcu_state)
              4'h0:
                display_brigthness[15:8] <= io_mcu_data;
              4'h1:
                display_brigthness[7:0] <= io_mcu_data;
            endcase
          end

        //Loading of the two bytes of the touch panel read address high byte first
        8'h3C:
          begin
            case(mcu_state)
              4'h0:
                touch_panel_address[15:8] <= io_mcu_data;
              4'h1:
                touch_panel_address[7:0] <= io_mcu_data;
            endcase
          end


      endcase
    end


//-------------------------------------------------------------------------------------
//For reading data the requested byte is loaded into a register on the positive edge
//of the i_mcu_clk signal. 

  always @(posedge i_mcu_clk)
    begin
      case(mcu_command)
        //Reading of the FPGA version needs loading of high byte first
        8'h06:
          begin
            case(mcu_state)
              4'h0:
                o_mcu_data <= 8'h14;
              4'h1:
                o_mcu_data <= 8'h32;
            endcase
          end

        //Reading of the touch panel address needs loading of high byte first
        8'h41:
          begin
            case(mcu_state)
              4'h0:
                o_mcu_data <= touch_panel_address[15:8];
              4'h1:
                o_mcu_data <= touch_panel_address[7:0];
            endcase
          end
      endcase
    end

//-------------------------------------------------------------------------------------
//Control of the bidirectional bus for input and output
//The bus direction is based on the read write signal

  assign io_mcu_data = i_mcu_rws ? 8'bZ : o_mcu_data;

//-------------------------------------------------------------------------------------
//Connect the relay control signals to the outside world

  assign o_relay1_1 = relay1_1;
  assign o_relay1_2 = relay1_2;
  assign o_relay1_3 = relay1_3;
  assign o_ac_dc_1  = ac_dc_1;

  assign o_relay2_1 = relay2_1;
  assign o_relay2_2 = relay2_2;
  assign o_relay2_3 = relay2_3;
  assign o_ac_dc_2  = ac_dc_2;

  assign o_adc1_encA = adc1_encA;
  assign o_adc1_encB = adc1_encB;
  assign o_adc2_encA = adc2_encA;
  assign o_adc2_encB = adc2_encB;

//-------------------------------------------------------------------------------------

endmodule

//-------------------------------------------------------------------------------------



